{$R-,V-,S-,B-,I-}                          {Turn off all the checking}
{$M 1024,100,100}                          {Small stack, no heap}

{$I OPDEFINE.INC}

program BPQLog;
   {-Utility for logging BPQ  operations}

uses
   Dos, OpInline, OpInt, OpTsr;

type
   String80 = string[80];

const
   BPQHandle = 16;                         {ISR handle for int 7F}
   ToggleLogKey = $0426;                   {Toggle logging ctrl L}

   {external functions}
   UnloadProgram = Ord('U');

   ModuleName : string[6] = 'BPQLOG';       {module name}
   Version    = '1.10';

   ProgName : String[31] = 'BPQLog '+Version+ ' BPQ Printer Logging';
   Copyright : String[37] = 'Written by Martin Schick, KA4IWG 1996';

   LoggingIsActive : Boolean = False;   {True while we're gathering log info}

var
   WatchCall : String[10];
   WatchPort : String[10];
   BPQBinByte : byte;
   Infer : Boolean;


procedure ToggleFlag(var Flag : Boolean);
                                           {-Toggle any log flag}
   begin
        PopupsOff;
        Flag := not Flag;
        PopupsOn;
   end;


{$F+}
procedure ToggleLogging(var Regs :Registers);
                                  {-Toggle logging - activated by <Alt><Tab>}
   begin
        ToggleFlag(LoggingIsActive);
        if LoggingIsActive then
              WriteLn('Logging is active')
        else
              WriteLn('Logging is inactive');
        end;


procedure BPQMonitor(BP : Word); interrupt;
                                           {-ISR called on every INT $7F}
   const
        CR : byte = $0D;
        LF : byte = $0A;

   var
        Regs : IntRegisters absolute BP;
        I : Integer;
        SaveRegs : IntRegisters;
        BPQbuff : Array[1..256] of byte;
        Vector : Pointer;
        Caller : Boolean;
        Porter : Boolean;
        Info : Boolean;


   procedure PrintChar(var OutChar : byte);
        begin
              if Caller or Porter or (Caller and Porter)
                        or ((WatchCall = '') and (WatchPort = '')) then
              begin
                  if ((WatchCall = '') and (WatchPort = '') and (not(Infer)))
                     or (Infer and Info) or not(Infer) then
                  begin
                        Regs.DX := 0;
                        Regs.AH := 0;
                        Regs.AL := OutChar;
                        EmulateInt(Regs, Vector);
                  end;
              end;
        end;


   procedure PrintString(var OutStr : String80);
        var
              Q : Integer;
              OutChar : byte;

        begin
              for Q := 1 to Length(OutStr) do
              begin
                  OutChar := Ord(OutStr[Q]);
                  PrintChar(OutChar);
              end;
        end;


   Procedure Moni;                  { Procedure to decode monitored packets }
        Var
              OutStr : String[80];
              St : String[10];
              I,J    : Integer;
              pass : Boolean;

{*************************** Start of Callsign ************************}
        Procedure Callsign(S:Integer);                   { Decode callsigns }
              Var
                  I : Integer;
                  MonCall : String[10];

              Begin
                  I := 1;
                  MonCall := '';
                  While (I<7) and ((BPQbuff[I+S] Shr 1) <> $20) do
                  Begin
                        OutStr := OutStr + Chr(BPQbuff[I+S] Shr 1);
                        MonCall := MonCall + Chr(BPQbuff[I+S] Shr 1);
                        I := I + 1;
                  End;

                  Str(((BPQbuff[S+7] Shr 1) and $0F),St);        {Strip SSID}
                  If St <> '0' then
                  begin
                        OutStr := OutStr + '-' +  St;
                        MonCall := MonCall + '-' + St;
                  end;
                  If MonCall = WatchCall then Caller := True;
              End;
{************************** Start of Netrom **************************}
        Procedure Netrom(S:Integer);
              Begin
                  PrintString(OutStr);
                  PrintChar(CR);
                  PrintChar(LF);
                  OutStr := '[Netrom data] ';
                  Callsign(S+2);
                  Outstr := OutStr + ' to ';
                  Callsign(S+9);

                  Case (BPQbuff[S+22] and $0F) of
                        1 : OutStr := OutStr + ' <Conn Req>';
                        2 : Begin
                            OutStr := OutStr + ' <Conn Ack>';
                            End;
                        3 : OutStr := OutSTr + ' <Disc Req>';
                        4 : OutStr := OutStr + ' <Disc Ack>';
                        5 : Begin
                            OutStr := OutStr + ' <Info>';
                            Info := True;
                            J := S + 20;     { Correct counter to show text }
                            End;
                        6 : OutStr := OutStr + ' <Info Ack>';
                  else
                        Begin
                            Str((BPQbuff[S+22] and $0F),St);
                            OutStr := OutStr + ' Type ' + St;
                        End
                  End;
              End;

        Procedure Node_table;
              Var
                  I,S : Integer;

              Begin
                  PrintString(OutStr);
                  PrintChar(CR);
                  PrintChar(LF);
                  OutStr := '';

                  S := 22;
                  I := 1;
                  While (I<7) and ((BPQbuff[I+S]) <> $20) do
                  Begin
                        OutStr := OutStr + Chr(BPQbuff[I+S]);
                        I := I + 1;
                  End;

                  OutStr := OutStr + '   ';
                  S := S + 6;
                  Repeat
                        OutStr := OutStr + 'Node: ';
                        Callsign(S);
                        S := S + 7;

                        OutStr := OutStr + '/';
                        I := 1;
                        While (I<7) and (BPQbuff[I+S] <> $20) do
                        Begin
                            OutStr := OutStr + Chr(BPQbuff[I+S]);
                            I := I + 1;
                        End;
                        S := S + 6;

                        OutStr := OutStr + ' via ';
                        Callsign(S);
                        S := S + 7;

                        Str(BPQbuff[S],St);
                        OutStr := OutStr + ' qual:' + st;
                        S := S + 1;
                        PrintString(OutStr);
                        PrintChar(CR);
                        PrintChar(LF);
                        OutStr := '          ';

                  Until S >= Regs.CX
              end;

{*************************** Start of Header **************************}
        Function Header:Boolean;
              Var
                  I : Integer;

              Begin
                  OutStr := '';
                  Info := False;
                  Caller := False;
                  Porter := False;

                  Callsign(12);                             { From callsign }

                  OutStr := OutStr + '>';

                  Callsign(5);                                { To callsign }

                  J := 19;
                  While (BPQbuff[J] and $01) <> 1 do
                  Begin
                        OutStr := OutStr + ',';
                        Callsign(J);                        { Digi callsign }
                        If (BPQbuff[J+7] and $80) = $80 then
                            OutStr := OutStr + '*';                {Digi bit}
                        J := J + 7;
                  End;

                  Str((BPQbuff[3] and $0F),St);               { Port number }
                  OutStr := OutStr + ' Port=' + St;
                  If St = WatchPort then Porter := True;

                  Case (BPQbuff[J+1] and $01) of
                  0 : Begin                            {An information frame}
                      Case BPQbuff[J+2] of               { Case on the PID }
                          $CF : Begin
                                OutStr := OutStr + ' [Net/Rom]';
                                Netrom(J);
                                End;
                          $F0 : Begin                       { Normal Packet }
                                OutStr := OutStr + ':';
                                Info := True;
                                End;
                           Else  Begin                      { Any other PID }
                                 Str(BPQbuff[J+2],St);
                                 Outstr := OutStr + ' PID '+ St;
                                 End;
                           End;                           { End of PID case }
                      End;
                  1 : Begin                        { Must be a U or S frame }
                      If (BPQbuff[J+1] and $02)=0 then
                                             { Is this an supervisory frame }
                      Begin
                          St := '';
                          Case (BPQbuff[J+1] and $0C) of
                               $00 : St := 'RR';
                               $04 : St := 'RNR';
                               $08 : St := 'REJ';
                          End;

                          OutStr := OutStr + ' <' + St;
                          Str((BPQbuff[J+1] Shr 5),St);    { Strip out N(R) }
                          OutStr := OutStr + ' R' + St + '>';
                      End
                      else
                          Case (BPQbuff[J+1] and $EC) of      { U Frame }
                          0  : Begin
                               OutStr := OutStr + ' <UI>';
                               If ((BPQbuff[6] Shr 1) = Ord('N')) and
                                  ((BPQbuff[7] Shr 1) = Ord('O')) and
                                  (BPQbuff[22] = $FF ) then
                                       Node_table
                               else
                                  Info := True;
                               End;
                          $0C : OutStr := OutStr + ' <DM>';
                          $2C : OutStr := OutStr + ' <SABM>';
                          $40 : OutStr := OutStr + ' <DISC>';
                          $60 : OutStr := OutStr + ' <UA>';
                          $84 : OutStr := OutStr + ' <FRMR>';
                          End;
                      End;
                  End;                             { End of Info/Super case }

                  PrintString(OutStr);

                  If Info then Header := True
                  else Header := False;
              End;
{**************************** End of Header ****************************}

        Begin
              If Header then                       { If valid info in frame }
              Begin
                  I := J + 3;
                  PrintChar(CR);
                  PrintChar(LF);
                  While I <= Regs.CX do
                  Begin
                        PrintChar(BPQbuff[I]);     { Write it out to screen }
                        If BPQbuff[I] = $0D then
                        begin
                            PrintChar(CR);
                            PrintChar(LF);
                        end;
                        I := I + 1;
                  End;
                  If BPQbuff[Regs.CX] <> $0D then
                  begin
                        PrintChar(CR);
                        PrintChar(LF);
                  end;
              End
              Else
                  PrintChar(CR);
                  PrintChar(LF);
        End;

   begin
        if LoggingIsActive and (Regs.AH = $B) then
        begin
              EmulateInt(Regs, IsrArray[BPQHandle].OrigAddr);

              if Regs.CX > 0 then
              begin
                  with Regs do
                  begin
                        SaveRegs := Regs;
                  end;

                  for I := 1 to Regs.CX do
                  begin
                        BPQbuff[I] :=  MemW[Regs.ES:Regs.DI];
                        Inc(Regs.DI);
                  end;

                  GetIntVec($17,Vector);
                  Moni;

                  with Regs do
                  begin
                        Regs := SaveRegs;
                  end;
              end;
              IntReturn(Regs);
        end
        else
              ChainInt(Regs, IsrArray[BPQHandle].OrigAddr)
        end;


procedure Unload(var Regs : Registers);
                                {-Unload from memory - activated by <Alt><U>}
   var
        Cur7F, SaveRetVec, P : Pointer;
   begin
        WriteLn('Unloading BPQLog from memory');
                                         {save copy of current return vector}
        SaveRetVec := GetReturnVec(BPQBinByte);

                                                      {set new return vector}
        SetVecOnReturn(BPQBinByte, IsrArray[BPQHandle].OrigAddr);

                                    {is the current INT $7F vector in OPTSR?}
        GetIntVec(BPQBinByte, Cur7F);
        P := @DisableTSR;
        if OS(Cur7F).S = OS(P).S then
                                      {make program's ISR the current vector}
              SetIntVec(BPQBinByte, IsrArray[BPQHandle].NewAddr);

                                                     {try to disable the TSR}
        if DisableTSR then
              begin

                  LoggingIsActive := False;

              end
        else
              begin
                                         {restore the previous return vector}
                  SetVecOnReturn(BPQBinByte, SaveRetVec);

                                                   {restore the previous ISR}
                  SetIntVec(BPQBinByte, Cur7F);
              end;
   end;
{$F-}


procedure ExternalIFC(BP : Word); interrupt;
                                {-This ISR interfaces with the outside world}
   var
        Regs : IntRegisters absolute BP;
        SavePSP : Word;
   begin
                                                              {interrupts on}
        InterruptsOn;
                                                            {turn popups off}
        PopupsOff;
        with Regs do
              case AH of
                  UnloadProgram :                             {unload BPQLOG}
                        begin
                                                           {save current PSP}
                        SavePSP := GetPSP;
                                                     {switch back to our PSP}
                        SetPSP(PrefixSeg);
                                                       {try to unload BPQLOG}
                        AL := Ord(DisableTSR);
                        if AL = 1 then begin
                            LoggingIsActive := False;
                        end;
                                                {switch back to previous PSP}
                        SetPSP(SavePSP);
                  end;
              end;
        PopupsOn;
   end;


procedure DisableResidentCopy(IP : IfcPtr);
                                      {-Disable resident copy of the program}
   var
        Regs : IntRegisters;
   begin
                                                  {restore interrupt vectors}
        RestoreAllVectors;
                                                     {make the function call}
        Regs.AH := UnloadProgram;
        EmulateInt(Regs, IP^.CmdEntryPtr);
        if Boolean(Regs.AL) then begin
              WriteLn(ModuleName, ' unloaded');
        end
        else
              WriteLn('Unable to unload ', ModuleName);
              Halt;
   end;


procedure Abort(Msg : String80);
                                                  {-Write a message and halt}
   begin
        WriteLn(Msg);
        Halt(1);
   end;

procedure ShowHelp;

   begin
        WriteLn('Usage: ',ModuleName,' [Options]'^M^J);
        WriteLn('Options:');
        WriteLn('/? or -?                this help screen');
        WriteLn('/U or -U                unload ',ModuleName);
        WriteLn('/I or -I                monitor info packets');
        WriteLn('/P=n                    monitor port # n');
        WriteLn('/C=aanaaa               monitor call aanaaa');
        WriteLn('<empty>                 monitor all ports/calls');
        WriteLn;
        WriteLn('<ctrl><L>               toggle monitoring off/on');
        Halt(0)
   end;


procedure Initialize;
                          {-Initialize and check for command line parameters}
   var
        I   : Word;
        Opt : String80;
        ModPtr : IfcPtr;


   procedure InvalidOption;
                                         {-displays error message and aborts}
        begin
              WriteLn(Opt, ' is an invalid option'^M^J);
              Halt(0);
        end;

   procedure GetWatch;
        var
              ZZ : integer;

        begin
              if UpCase(Opt[2]) = 'C' then
                  begin
                  if Length(Opt) > 12 then InvalidOption;
                  WatchCall := Copy(Opt, 4, (Length(Opt)-3));
                  for ZZ := 1 to Length(WatchCall) do
                        WatchCall[ZZ] := UpCase(WatchCall[ZZ]);
                        WriteLn(' Monitoring callsign ',WatchCall);
                  end
              else
                  begin
                  if Length(Opt) > 5 then InvalidOption;
                  WatchPort := Copy(Opt, 4, (Length(Opt)-3));
                  for ZZ := 1 to Length(WatchPort) do
                        WatchPort[ZZ] := UpCase(WatchPort[ZZ]);
                        WriteLn(' Monitoring port #',WatchPort);
                  end;
              end;


   begin
                                              {see if we're already resident}
        ModPtr := ModulePtrByName(ModuleName);

        WatchPort := '';
        WatchCall := '';
        Infer := False;

        for I := 1 to ParamCount do begin
              Opt := ParamStr(I);
              if ((Opt[1] = '/') or (Opt[1] = '-')) and (Length(Opt) >= 2) then
                  case UpCase(Opt[2]) of

                        '?' : ShowHelp;

                        'C' : GetWatch;

                        'P' : GetWatch;

                        'I' : Infer := True;

                        'U' : if ModPtr = nil then
                                 Abort('Program is not resident. Cannot unload.')
                              else
                                 DisableResidentCopy(ModPtr);
                  else InvalidOption;
              end
              else
                  InvalidOption;
              end;
        if Infer then WriteLn(' Monitoring information frames only ');

                                                          {abort if we're already resident}
        if ModPtr <> nil then
        Abort('Program is already installed');
   end;

procedure CheckBPQInterrupt;
   var
        BPQConfig : File of byte;
        NN : integer;
        Vex : Pointer;
        Rex : Registers;

   begin
        Assign(BPQConfig, 'BPQCFG.BIN');
        {$I-}reset(BPQConfig);{$I+}
        NN := IOResult;
        if NN<>0 then Abort('BPQCFG.BIN not found');

        Seek(BPQConfig, 70);
        Read(BPQConfig, BPQBinByte);
        WriteLn('BPQ utilizes interrupt number ',BPQBinByte);
        WriteLn;

        GetIntVec(BPQBinByte, Vex);
        if PtrToLong(Vex) = 0 then Abort('BPQ Code is not loaded');

        Rex.AH := 0;
        Rex.AL := 1;
        Intr(BPQBinByte, Rex);
        if (Chr(Rex.AL)<>'B') or (Chr(Rex.AH)<>'P')
        or (Chr(Rex.BL)<>'Q') or (Chr(Rex.BH)<>' ') then
              Abort('BPQ Code is not loaded');

        Rex.AH := 2;
        Rex.DX := 0;
        Intr($17, Rex);
        if ($8 and Rex.AH) = 8 then
        begin
           WriteLn('Printer is not responding!');
           Abort('Please make sure it is on and try again...');
        end;
   end;


begin
                                                             {signon message}
   WriteLn(ProgName, ^M^J, Copyright, ^M^J);

                                          {interpret command line parameters}
   Initialize;
   CheckBPQInterrupt;

                                                         {install our module}
   InstallModule(ModuleName, @ExternalIFC);
   WriteLn('   Program installed');

                                 {Grab interrupt vectors and set up hot keys}
   if not InitVector(BPQBinByte, BPQHandle, @BPQMonitor) or
      not DefinePop(ToggleLogKey, ToggleLogging, Ptr(SSeg, SPtr), True) then
              Abort('Unable to install BPQLOG');

                                            {Activate popups and go resident}
   PopupsOn;
   StayRes(ParagraphsToKeep, 0);

                                                  {if we get here we failed}
   Abort('Unable to go resident');
end.
