
{$R-,V-,S-,B-,I-}                 {Turn off all the checking}
{$M 1024,100,100}                 {Small stack, no heap}

{$I OPDEFINE.INC}

program BPQNBlok;          {Utility for blocking DX spots form a series of nodes}

uses
   Dos, OpInline, OpInt, OpTsr;

type
   String80 = string[80];

const
   BPQNBlokHandle = 16;                     {ISR handle for int 7F}
   ToggleScrnKey = $042D;                  {Toggle screening CTRL-X}

   {external functions}
   UnloadProgram = Ord('U');

   ModuleName : string[8] = 'BPQNBlok';     {module name}
   Version    = '1.00';

   ProgName : String[37] = 'BPQNBlok '+Version+ ' BPQ Node Filter Routine';
   Copyright : String[37] = 'Written by Martin Schick, KA4IWG 1996';

   ScreeningIsActive : Boolean = True;    {Screening is on at startup}

   PCPattern : Array[1..5] of byte = (80, 67, 49, 49, 94); {PC11 pattern}


var
   Pattern : Array[1..16,1..15] of byte;       {Node block callsign patterns}
   PatternCount : Array[1..16] of integer;     {Size of callsigns}
   NodeCount : integer;                        {Number of nodes}
   BPQBinByte : byte;
   Opt : String80;

procedure ToggleFlag(var Flag : Boolean);  {Toggle any flag}
   begin
      PopupsOff;                              {Shut off popups}
      Flag := not Flag;
      PopupsOn;                             {Turn popups back on}
   end;


{$F+}
procedure ToggleScreening(var Regs :Registers);
                               {Toggle screening - activated by <Ctrl><X>}
   begin
      ToggleFlag(ScreeningIsActive);
      if ScreeningIsActive then
          Writeln(' Node blocking is active')
      else
          Writeln(' Node blocking is inactive');
   end;


procedure BPQScreen(BP : Word); interrupt; {ISR called on every INT $7F}
   type
      str255 = string[255];
   var
      Regs : IntRegisters absolute BP;
      SaveRegs : IntRegisters;
      Vector : Pointer;
      I, J, K : Integer;
      Match : Boolean;
      Latch : Boolean;
      Hatch : Boolean;
      Offset : Word;

   begin
      if ScreeningIsActive and (Regs.AH = $03) then
          begin
              Match := False;
              Latch := False;
              EmulateInt(Regs, IsrArray[BPQNBlokHandle].OrigAddr);

              if Regs.CX > 0 then
              begin
                  with Regs do
                  begin
                      SaveRegs := Regs;
                  end;

                  for I := 0 to Regs.CX do        {Find PC11 pattern}
                  begin
                    if not Match then
                    begin
                      if PCPattern[1] = Mem[Regs.ES : Regs.DI] then
                      begin
                      {Check rest of Pattern}
                          Match := True;
                          Offset := Regs.DI;
                          for J := 1 to 5 do
                          begin
                              if PCPattern[J] <> Mem[Regs.ES : Offset] then Match := False;
                              Inc(Offset);
                          end;
                      end;
                    end;

                    if not Latch then         {Find nodes in list}
                    begin
                      for K := 1 to NodeCount do
                          if Pattern[K,1] = Mem[Regs.ES : Regs.DI] then
                          begin
                          {Check for node call}
                              Hatch := True;
                              Offset := Regs.DI;
                              for J := 1 to PatternCount[K] do
                              begin
                                  if Pattern[K,J] <> Mem[Regs.ES : Offset] then Hatch := False;
                                  Inc(Offset);
                              end;
                              if Hatch then Latch := True; {Node was in list}
                          end;

                    end;
                    Inc(Regs.DI);
                  end;

              with Regs do
              begin
                  Regs := SaveRegs;
                  if Match and Latch then Regs.CX := 0;
              end;
          end;
          IntReturn(Regs);
      end
      else
          ChainInt(Regs, IsrArray[BPQNBlokHandle].OrigAddr)
   end;


procedure Unload(var Regs : Registers);
                                {Unload from memory - activated by <Ctrl><U>}
   var
   Cur7F, SaveRetVec, P : Pointer;

   begin
      WriteLn('Unloading BPQNBlok from memory');
                                         {save copy of current return vector}
      SaveRetVec := GetReturnVec(BPQBinByte);

                                           {set new return vector}
      SetVecOnReturn(BPQBinByte, IsrArray[BPQNBlokHandle].OrigAddr);

                                    {is the current INT $7F vector in OPTSR?}
      GetIntVec(BPQBinByte, Cur7F);
      P := @DisableTSR;
      if OS(Cur7F).S = OS(P).S then
                                      {make program's ISR the current vector}
          SetIntVec(BPQBinByte, IsrArray[BPQNBlokHandle].NewAddr);

                                           {try to disable the TSR}
          if DisableTSR then begin

              ScreeningIsActive := False;

          end
      else begin
                                         {restore the previous return vector}
          SetVecOnReturn(BPQBinByte, SaveRetVec);

                                           {restore the previous ISR}
          SetIntVec(BPQBinByte, Cur7F);
      end;
   end;
{$F-}


procedure ExternalIFC(BP : Word); interrupt;
                                 {This ISR interfaces with the outside world}
   var
      Regs : IntRegisters absolute BP;
      SavePSP : Word;

   begin
                                           {interrupts on}
      InterruptsOn;

                                          {turn popups off}
      PopupsOff;

      with Regs do
      case AH of
          UnloadProgram :                  {unload BPQNBlok}
          begin
                                           {save current PSP}
              SavePSP := GetPSP;

                                           {switch back to our PSP}
              SetPSP(PrefixSeg);

                                           {try to unload BPQNBlok}
              AL := Ord(DisableTSR);
              if AL = 1 then begin

                  ScreeningIsActive := False;
              end;

                                           {switch back to previous PSP}
              SetPSP(SavePSP);
          end;
      end;

      PopupsOn;
   end;


procedure DisableResidentCopy(IP : IfcPtr);
                                      {-Disable resident copy of the program}
   var
      Regs : IntRegisters;
   begin
                                           {restore interrupt vectors}
      RestoreAllVectors;

                                           {make the function call}
      Regs.AH := UnloadProgram;
      EmulateInt(Regs, IP^.CmdEntryPtr);
      if Boolean(Regs.AL) then begin
          WriteLn(ModuleName, ' unloaded');
      end
      else
          WriteLn('Unable to unload ', ModuleName);

          Halt;
   end;


procedure Abort(Msg : String80);
                                           {-Write a message and halt}
   begin
      WriteLn(Msg);
      Halt(1);
   end;

procedure ShowHelp;                        {Help screen}

   begin
        WriteLn('Usage: ',ModuleName,' [Options]'^M^J);
        WriteLn('Options:');
        WriteLn('No options are needed to start.  NodeBlok.Dat must exist');
        WriteLn('/? or -?                      this help screen');
        WriteLn('/U or -U                      unload ',ModuleName);
        WriteLn;
        WriteLn('<ctrl><X>       used to toggle screening off/on');
        Halt(0)
   end;


procedure Initialize(var Opt : String80);
                          {-Initialize and check for command line parameters}
   var
      ModPtr : IfcPtr;


   procedure InvalidOption;
                                         {-displays error message and aborts}
      begin
          WriteLn(Opt, ' is an invalid option'^M^J);
          WriteLn('Enter BPQNBlok /? for help');
          Halt(0);
      end;


   begin
                                           {see if we're already resident}
      ModPtr := ModulePtrByName(ModuleName);

        if ParamCount = 1 then
          if ((Opt[1] = '/') or (Opt[1] = '-')) and (Length(Opt) >= 2) then
            case UpCase(Opt[2]) of

              '?' : ShowHelp;

              'U' : if ModPtr = nil then
                        Abort('Program is not resident. Cannot unload.')
                    else
                        DisableResidentCopy(ModPtr);
            else InvalidOption;
            end

          else
            InvalidOption;


                                           {abort if we're already resident}
      if ModPtr <> nil then
          Abort('Program is already installed');

                                           {install our module}
          InstallModule(ModuleName, @ExternalIFC);
          WriteLn('     Program installed');
   end;

   procedure CheckBPQInterrupt;
      var
          BPQConfig : File of byte;
          NN : integer;
          Vex : Pointer;
          Rex : Registers;

      begin                                {Try to open BPQCFG.BIN}
          Assign(BPQConfig, 'BPQCFG.BIN');
          {$I-}reset(BPQConfig);{$I+}
          NN := IOResult;
          if NN<>0 then Abort('BPQCFG.BIN not found');

          Seek(BPQConfig, 70);             {Find which INT BPQ is using}
          Read(BPQConfig, BPQBinByte);
          WriteLn('BPQ utilizes interrupt number ',BPQBinByte);
          WriteLn;
                                           {Check that vector for address}
          GetIntVec(BPQBinByte, Vex);
          if PtrToLong(Vex) = 0 then Abort('BPQ Code is not loaded');

          Rex.AH := 0;                     {Verify that the INT is BPQ}
          Rex.AL := 1;
          Intr(BPQBinByte, Rex);
          if (Chr(Rex.AL)<>'B') or (Chr(Rex.AH)<>'P')
             or (Chr(Rex.BL)<>'Q') or (Chr(Rex.BH)<>' ') then
                Abort('BPQ Code is not loaded');

      end;

   procedure GetNodes;
     var
        NodeFile : Text;
        NodeCall : String80;
        NN : integer;
        I : integer;

     begin
        Assign(NodeFile,'NODEBLOK.DAT');
        {$I-}reset(NodeFile);{$I+}
        NN := IOResult;
        if NN<>0 then Abort('Can not open NodeBlok.Dat file');

        NN := 1;
        while (not EOF(NodeFile)) and (NN < 17) do     {16 Nodes allowed}
            begin
              Readln(NodeFile,NodeCall);               {Callsigns up to 12 chars}
              if Length(NodeCall) > 12 then Delete(NodeCall,13,Length(NodeCall)-12);
              Writeln(NN,'  ',NodeCall);

              Pattern[NN,1] := 94;                     {Add ^}
              for I := 1 to Length(NodeCall) do Pattern[NN,I+1] := Ord(UpCase(NodeCall[I]));
              PatternCount[NN] := Length(NodeCall)+3;
              Pattern[NN,PatternCount[NN]-1] := 94;    {Add ^}
              Pattern[NN,PatternCount[NN]] := 72;      {Add H}
              NN := NN+1
            end;
        NodeCount := NN-1;
        Close(NodeFile);
     end;

begin
                                           {signon message}
   WriteLn(ProgName, ^M^J, Copyright, ^M^J);

                                          {interpret command line parameters}
   if ParamCount > 1 then Abort(' Only one parameter needed on command line!');
   if ParamCount = 1 then Opt := ParamStr(1);

   CheckBPQInterrupt;                      {Verify BPQ is up and running}
   GetNodes;
   Initialize(Opt);                        {Determine what is requested}


                                 {Grab interrupt vectors and set up hot keys}
   if not InitVector(BPQBinByte, BPQNBlokHandle, @BPQScreen) or
    not DefinePop(ToggleScrnKey, ToggleScreening, Ptr(SSeg, SPtr), True) then
          Abort('Unable to install BPQNBlok');

                                           {Activate popups and go resident}
   PopupsOn;
   StayRes(ParagraphsToKeep, 0);

                                           {if we get here we failed}
   Abort('Unable to go resident');
end.
