
{$R-,V-,S-,B-,I-}                 {Turn off all the checking}
{$M 1024,100,100}                 {Small stack, no heap}

{$I OPDEFINE.INC}

program BPQScrn;           {Utility for screening input from BPQ operations}

uses
   Dos, OpInline, OpInt, OpTsr;

type
   String80 = string[80];

const
   BPQScrnHandle = 16;                     {ISR handle for int 7F}
   ToggleScrnKey = $041F;                  {Toggle screening ctrl S}

   {external functions}
   UnloadProgram = Ord('U');

   ModuleName : string[7] = 'BPQSCRN';     {module name}
   Version    = '1.10';

   ProgName : String[34] = 'BPQScrn '+Version+ ' BPQ Screening Routine';
   Copyright : String[37] = 'Written by Martin Schick, KA4IWG 1996';

   ScreeningIsActive : Boolean = False;    {Screening is off at startup}

var
   Pattern : Array[1..40] of byte;         {Byte pattern to screen}
   PatternCount : integer;
   BPQBinByte : byte;
   Opt : String80;

procedure ToggleFlag(var Flag : Boolean);  {Toggle any flag}
   begin
      PopupsOff;                              {Shut off popups}
      Flag := not Flag;
      PopupsOn;                             {Turn popups back on}
   end;


{$F+}
procedure ToggleScreening(var Regs :Registers);
                               {Toggle screening - activated by <Ctrl><Tab>}
   begin
      ToggleFlag(ScreeningIsActive);
      if ScreeningIsActive then
          Writeln(' Screening is active')
      else
          Writeln(' Screening is inactive');
   end;


procedure BPQScreen(BP : Word); interrupt; {ISR called on every INT $7F}
   type
      str255 = string[255];
   var
      Regs : IntRegisters absolute BP;
      SaveRegs : IntRegisters;
      Vector : Pointer;
      I, J : Integer;
      Match : Boolean;
      Offset : Word;

   begin
      if ScreeningIsActive and (Regs.AH = $03) then
          begin
              EmulateInt(Regs, IsrArray[BPQScrnHandle].OrigAddr);

              if Regs.CX > 0 then
              begin
                  with Regs do
                  begin
                      SaveRegs := Regs;
                  end;

                  for I := 0 to Regs.CX do
                  begin
                      if Pattern[1] = Mem[Regs.ES : Regs.DI] then
                      begin
                      {Check rest of Pattern}
                          Match := True;
                          Offset := Regs.DI;
                          for J := 1 to PatternCount do
                          begin
                              if Pattern[J] <> Mem[Regs.ES : Offset] then Match := False;
                              Inc(Offset);
                          end;

                          if Match then
                          begin
                              Offset := Regs.DI;
                              for J := 1 to PatternCount do
                              begin
                                  Mem[Regs.ES : Offset] := $20;
                                  Inc(Offset);
                              end;
                          end;
                  end;
                  Inc(Regs.DI);
              end;

              with Regs do
              begin
                  Regs := SaveRegs;
              end;
          end;
          IntReturn(Regs);
      end
      else
          ChainInt(Regs, IsrArray[BPQScrnHandle].OrigAddr)
   end;


procedure Unload(var Regs : Registers);
                                {Unload from memory - activated by <Ctrl><U>}
   var
   Cur7F, SaveRetVec, P : Pointer;

   begin
      WriteLn('Unloading BPQScrn from memory');
                                         {save copy of current return vector}
      SaveRetVec := GetReturnVec(BPQBinByte);

                                           {set new return vector}
      SetVecOnReturn(BPQBinByte, IsrArray[BPQScrnHandle].OrigAddr);

                                    {is the current INT $7F vector in OPTSR?}
      GetIntVec(BPQBinByte, Cur7F);
      P := @DisableTSR;
      if OS(Cur7F).S = OS(P).S then
                                      {make program's ISR the current vector}
          SetIntVec(BPQBinByte, IsrArray[BPQScrnHandle].NewAddr);

                                           {try to disable the TSR}
          if DisableTSR then begin

              ScreeningIsActive := False;

          end
      else begin
                                         {restore the previous return vector}
          SetVecOnReturn(BPQBinByte, SaveRetVec);

                                           {restore the previous ISR}
          SetIntVec(BPQBinByte, Cur7F);
      end;
   end;
{$F-}


procedure ExternalIFC(BP : Word); interrupt;
                                 {This ISR interfaces with the outside world}
   var
      Regs : IntRegisters absolute BP;
      SavePSP : Word;

   begin
                                           {interrupts on}
      InterruptsOn;

                                          {turn popups off}
      PopupsOff;

      with Regs do
      case AH of
          UnloadProgram :                  {unload BPQSCRN}
          begin
                                           {save current PSP}
              SavePSP := GetPSP;

                                           {switch back to our PSP}
              SetPSP(PrefixSeg);

                                           {try to unload BPQSCRN}
              AL := Ord(DisableTSR);
              if AL = 1 then begin

                  ScreeningIsActive := False;
              end;

                                           {switch back to previous PSP}
              SetPSP(SavePSP);
          end;
      end;

      PopupsOn;
   end;


procedure DisableResidentCopy(IP : IfcPtr);
                                      {-Disable resident copy of the program}
   var
      Regs : IntRegisters;
   begin
                                           {restore interrupt vectors}
      RestoreAllVectors;

                                           {make the function call}
      Regs.AH := UnloadProgram;
      EmulateInt(Regs, IP^.CmdEntryPtr);
      if Boolean(Regs.AL) then begin
          WriteLn(ModuleName, ' unloaded');
      end
      else
          WriteLn('Unable to unload ', ModuleName);

          Halt;
   end;


procedure Abort(Msg : String80);
                                           {-Write a message and halt}
   begin
      WriteLn(Msg);
      Halt(1);
   end;


procedure ShowHelp;                        {Help screen}

   begin
        WriteLn('Usage: ',ModuleName,' [Options]'^M^J);
        WriteLn('Options:');
        WriteLn('/? or -?                      this help screen');
        WriteLn('/U or -U                      unload ',ModuleName);
        WriteLn('/$##$##$## or -$##$##$##      load ',ModuleName);
        WriteLn('    where ## are the ASCII code for the bytes to');
        WriteLn('    screen in hexadecimal.  Up to 20 characters');
        WriteLn('    can be entered, but must be seperated by $');
        WriteLn;
        WriteLn('<ctrl><S>       used to toggle screening off/on');
        Halt(0)
   end;


procedure Initialize(var Opt : String80);
                          {-Initialize and check for command line parameters}
   var
      ModPtr : IfcPtr;


   procedure InvalidOption;
                                         {-displays error message and aborts}
      begin
          WriteLn(Opt, ' is an invalid option'^M^J);
          WriteLn('Enter BPQSCRN /? for help');
          Halt(0);
      end;


   procedure ScreenPattern;
      var
          KK, JJ : integer;
          QQ : byte;
          HexSet1, HexSet2 : set of char;

      begin
          HexSet1 := ['0','1','2','3','4','5','6','7','8','9'];
          HexSet2 := ['A','B','C','D','E','F'];

          for KK := 1 to Length(Opt) do Opt[KK] := UpCase(Opt[KK]);

          JJ := pos('$',Opt) + 1;
          KK := 1;

          repeat
              if (JJ+1) > Length(Opt) then InvalidOption;

              if Opt[JJ] in HexSet1 then QQ := (Ord(Opt[JJ]) - 48) * 16
              else
                  if Opt[JJ] in HexSet2 then QQ := (Ord(Opt[JJ]) - 55) * 16
                  else InvalidOption;

              if Opt[JJ+1] in HexSet1 then QQ := QQ + (Ord(Opt[JJ+1]) - 48)
              else
                  if Opt[JJ+1] in HexSet2 then QQ := QQ + (Ord(Opt[JJ+1]) - 55)
                  else InvalidOption;

              Pattern[KK] := QQ;
              KK := KK + 1;
              JJ := JJ + 3;

          until JJ >= Length(Opt);

          PatternCount := KK - 1;
          write(' Searching for ');
          for JJ := 1 to PatternCount do write(Chr(Pattern[JJ]));
          writeln;

      end;


   begin
                                           {see if we're already resident}
      ModPtr := ModulePtrByName(ModuleName);

          if ((Opt[1] = '/') or (Opt[1] = '-')) and (Length(Opt) >= 2) then
          case UpCase(Opt[2]) of

              '$' : ScreenPattern;

              '?' : ShowHelp;

              'U' : if ModPtr = nil then
                        Abort('Program is not resident. Cannot unload.')
                    else
                        DisableResidentCopy(ModPtr);
          else InvalidOption;
          end

      else
          InvalidOption;


                                           {abort if we're already resident}
      if ModPtr <> nil then
          Abort('Program is already installed');

                                           {install our module}
          InstallModule(ModuleName, @ExternalIFC);
          WriteLn('     Program installed');
   end;

   procedure CheckBPQInterrupt;
      var
          BPQConfig : File of byte;
          NN : integer;
          Vex : Pointer;
          Rex : Registers;

      begin                                {Try to open BPQCFG.BIN}
          Assign(BPQConfig, 'BPQCFG.BIN');
          {$I-}reset(BPQConfig);{$I+}
          NN := IOResult;
          if NN<>0 then Abort('BPQCFG.BIN not found');

          Seek(BPQConfig, 70);             {Find which INT BPQ is using}
          Read(BPQConfig, BPQBinByte);
          WriteLn('BPQ utilizes interrupt number ',BPQBinByte);
          WriteLn;
                                           {Check that vector for address}
          GetIntVec(BPQBinByte, Vex);
          if PtrToLong(Vex) = 0 then Abort('BPQ Code is not loaded');

          Rex.AH := 0;                     {Verify that the INT is BPQ}
          Rex.AL := 1;
          Intr(BPQBinByte, Rex);
          if (Chr(Rex.AL)<>'B') or (Chr(Rex.AH)<>'P')
             or (Chr(Rex.BL)<>'Q') or (Chr(Rex.BH)<>' ') then
                Abort('BPQ Code is not loaded');

      end;



begin
                                           {signon message}
   WriteLn(ProgName, ^M^J, Copyright, ^M^J);

                                          {interpret command line parameters}
   if ParamCount = 0 then Abort(' Must supply parameters. Use /? for help');
   if ParamCount > 1 then Abort(' Only one parameter needed on command line!');
   Opt := ParamStr(1);

   CheckBPQInterrupt;                      {Verify BPQ is up and running}
   Initialize(Opt);                        {Determine what is requested}


                                 {Grab interrupt vectors and set up hot keys}
   if not InitVector(BPQBinByte, BPQScrnHandle, @BPQScreen) or
      not DefinePop(ToggleScrnKey, ToggleScreening, Ptr(SSeg, SPtr), True) then
          Abort('Unable to install BPQScrn');

                                           {Activate popups and go resident}
   PopupsOn;
   StayRes(ParagraphsToKeep, 0);

                                           {if we get here we failed}
   Abort('Unable to go resident');
end.
