
{$R-,V-,S-,B-,I-}                 {Turn off all the checking}
{$M 1024,100,100}                 {Small stack, no heap}

{$I OPDEFINE.INC}

program BPQSloDX;                 {Utility for blocking slow DX spots}

uses
   Dos, OpInline, OpInt, OpTsr;

type
   String80 = string[80];

const
   BPQSloDXHandle = 18;                     {ISR handle for int 7F}
   ToggleScrnKey = $0414;                   {Toggle screening ctrl-T}

   {external functions}
   UnloadProgram = Ord('U');

   ModuleName : string[8] = 'BPQSloDX';     {module name}
   Version    = '1.10';

   ProgName : String[47] = 'BPQSloDX '+Version+ ' BPQ Slow DX Spot Blocking Routine';
   Copyright : String[37] = 'Written by Martin Schick, KA4IWG 1996';

   ScreeningIsActive : Boolean = False;    {Screening is off at startup}

   PCPattern : Array[1..5] of byte = (80, 67, 49, 49, 94); {PC11 pattern}
   Carat : Byte = 94;


var
   Pattern : Byte;                    {Zulu time digit}
   Delay : Integer;                   {lag time in minutes}
   BPQBinByte : Byte;
   Opt : String80;

procedure ToggleFlag(var Flag : Boolean);  {Toggle any flag}
   begin
      PopupsOff;                              {Shut off popups}
      Flag := not Flag;
      PopupsOn;                             {Turn popups back on}
   end;


{$F+}
procedure ToggleScreening(var Regs :Registers);
                               {Toggle screening - activated by <ATL><T>}
   begin
      ToggleFlag(ScreeningIsActive);
      if ScreeningIsActive then
          Writeln(' Lag time on DX spots is active')
      else
          Writeln(' Lag time on DX spots is inactive');
   end;


procedure BPQScreen(BP : Word); interrupt; {ISR called on every INT $7F}
   type
      str255 = string[255];
   var
      Regs : IntRegisters absolute BP;
      SaveRegs : IntRegisters;
      I, J : Integer;
      Carats, SpotHr, SpotMin : Integer;
      Hours, Minutes, Seconds, Hundreths : Word;
      Match : Boolean;
      Latch : Boolean;
      Offset : Word;

   begin
      if ScreeningIsActive and (Regs.AH = $03) then
          begin
              Match := False;
              Latch := True;
              Carats := 0;

              EmulateInt(Regs, IsrArray[BPQSloDXHandle].OrigAddr);

              if Regs.CX > 0 then
              begin
                  with Regs do
                  begin
                      SaveRegs := Regs;
                  end;

                  for I := 0 to Regs.CX do
                  begin
                    if not Match then
                    begin
                      if PCPattern[1] = Mem[Regs.ES : Regs.DI] then  {PC11 pattern?}
                      begin
                      {Check rest of Pattern}
                          Match := True;
                          Offset := Regs.DI;
                          for J := 1 to 5 do
                          begin
                              if PCPattern[J] <> Mem[Regs.ES : Offset] then Match := False;
                              Inc(Offset);
                          end;
                      end;
                    end;
                    if Match then                         {A PC11 packet - DX spot}
                    begin
                      if Carat = Mem[Regs.ES : Regs.DI] then
                      begin
                      {Check for spot time}
                          Inc(Carats);
                          if Carats = 4 then                    {Time slot begins at 4th ^}
                          begin
                               Offset := Regs.DI;
                               Inc(Offset);                     {Strip out the carat}
                               Pattern := Mem[Regs.ES : Offset];
                               SpotHr := (Pattern - 48) * 10;     {Get the hour}
                               Inc(Offset);
                               Pattern := Mem[Regs.ES : Offset];
                               SpotHr := SpotHr + (Pattern-48);
                               Inc(Offset);
                               Pattern := Mem[Regs.ES : Offset];
                               SpotMin := (Pattern-48) * 10;      {Get the minute}
                               Inc(Offset);
                               Pattern := Mem[Regs.ES : Offset];
                               SpotMin := SpotMin + (Pattern-48);

                               SpotMin := SpotMin + Delay;        {Add in the lag delay}
                               if SpotMin > 59 then
                               begin
                                    Inc(SpotHr);                  {Adjust for 60 minutes}
                                    SpotMin := SpotMin - 60;
                               end;

                               GetTime(Hours, Minutes, Seconds, Hundreths);   {System time}

                               if (SpotHr > 23) and (Hours <> 23) then SpotHr := 0; {Adjust for 24 hours}
                               if SpotHr > Hours then Latch := False               {Check the lag}
                                  else
                                  if SpotMin >= Minutes then Latch := False;
                          end;
                      end;
                    end;
                  Inc(Regs.DI);
              end;

              with Regs do
              begin
                  Regs := SaveRegs;
                  if Match and Latch then Regs.CX := 0;    {Blank spot if over lag time}
              end;
          end;
          IntReturn(Regs);
      end
      else
          ChainInt(Regs, IsrArray[BPQSloDXHandle].OrigAddr)  {Screening inactive}
   end;


procedure Unload(var Regs : Registers);
                                {Unload from memory - activated by <Ctrl><U>}
   var
   Cur7F, SaveRetVec, P : Pointer;

   begin
      WriteLn('Unloading BPQSloDX from memory');
                                         {save copy of current return vector}
      SaveRetVec := GetReturnVec(BPQBinByte);

                                           {set new return vector}
      SetVecOnReturn(BPQBinByte, IsrArray[BPQSloDXHandle].OrigAddr);

                                    {is the current INT $7F vector in OPTSR?}
      GetIntVec(BPQBinByte, Cur7F);
      P := @DisableTSR;
      if OS(Cur7F).S = OS(P).S then
                                      {make program's ISR the current vector}
          SetIntVec(BPQBinByte, IsrArray[BPQSloDXHandle].NewAddr);

                                           {try to disable the TSR}
          if DisableTSR then begin

              ScreeningIsActive := False;

          end
      else begin
                                         {restore the previous return vector}
          SetVecOnReturn(BPQBinByte, SaveRetVec);

                                           {restore the previous ISR}
          SetIntVec(BPQBinByte, Cur7F);
      end;
   end;
{$F-}


procedure ExternalIFC(BP : Word); interrupt;
                                 {This ISR interfaces with the outside world}
   var
      Regs : IntRegisters absolute BP;
      SavePSP : Word;

   begin
                                           {interrupts on}
      InterruptsOn;

                                          {turn popups off}
      PopupsOff;

      with Regs do
      case AH of
          UnloadProgram :                  {unload BPQSloDX}
          begin
                                           {save current PSP}
              SavePSP := GetPSP;

                                           {switch back to our PSP}
              SetPSP(PrefixSeg);

                                           {try to unload BPQSloDX}
              AL := Ord(DisableTSR);
              if AL = 1 then begin

                  ScreeningIsActive := False;
              end;

                                           {switch back to previous PSP}
              SetPSP(SavePSP);
          end;
      end;

      PopupsOn;
   end;


procedure DisableResidentCopy(IP : IfcPtr);
                                      {-Disable resident copy of the program}
   var
      Regs : IntRegisters;
   begin
                                           {restore interrupt vectors}
      RestoreAllVectors;

                                           {make the function call}
      Regs.AH := UnloadProgram;
      EmulateInt(Regs, IP^.CmdEntryPtr);
      if Boolean(Regs.AL) then begin
          WriteLn(ModuleName, ' unloaded');
      end
      else
          WriteLn('Unable to unload ', ModuleName);

          Halt;
   end;


procedure Abort(Msg : String80);
                                           {-Write a message and halt}
   begin
      WriteLn(Msg);
      Halt(1);
   end;

procedure DelayTime;                    {Capture the spot delay time}
   var I : integer;

   begin
        Delete(Opt,1,2);
        Val(Opt, Delay, I);
        if I <> 0 then Abort('Unable to evaluate delay time');
        if Delay > 60 then Delay := 60;
   end;

procedure ShowHelp;                        {Help screen}

   begin
        WriteLn('Usage: ',ModuleName,' [Options]'^M^J);
        WriteLn('Options:');
        WriteLn('/$## or -$##                  lag time allowed, in minutes');
        WriteLn('/? or -?                      this help screen');
        WriteLn('/U or -U                      unload ',ModuleName);
        WriteLn;
        WriteLn('<alt><T>           used to toggle screening off/on');
        Halt(0)
   end;


procedure Initialize(var Opt : String80);
                          {-Initialize and check for command line parameters}
   var
      ModPtr : IfcPtr;


   procedure InvalidOption;
                                         {-displays error message and aborts}
      begin
          WriteLn(Opt, ' is an invalid option'^M^J);
          WriteLn('Enter BPQSloDX /? for help');
          Halt(0);
      end;


   begin
                                           {see if we're already resident}
      ModPtr := ModulePtrByName(ModuleName);

          if ((Opt[1] = '/') or (Opt[1] = '-')) and (Length(Opt) >= 2) then
          case UpCase(Opt[2]) of

              '$' : DelayTime;

              '?' : ShowHelp;

              'U' : if ModPtr = nil then
                        Abort('Program is not resident. Cannot unload.')
                    else
                        DisableResidentCopy(ModPtr);
          else InvalidOption;
          end

      else
          InvalidOption;


                                           {abort if we're already resident}
      if ModPtr <> nil then
          Abort('Program is already installed');

                                           {install our module}
          InstallModule(ModuleName, @ExternalIFC);
          WriteLn('     Program installed');
   end;

   procedure CheckBPQInterrupt;
      var
          BPQConfig : File of byte;
          NN : integer;
          Vex : Pointer;
          Rex : Registers;

      begin                                {Try to open BPQCFG.BIN}
          Assign(BPQConfig, 'BPQCFG.BIN');
          {$I-}reset(BPQConfig);{$I+}
          NN := IOResult;
          if NN<>0 then Abort('BPQCFG.BIN not found');

          Seek(BPQConfig, 70);             {Find which INT BPQ is using}
          Read(BPQConfig, BPQBinByte);
          WriteLn('BPQ utilizes interrupt number ',BPQBinByte);
          WriteLn;
                                           {Check that vector for address}
          GetIntVec(BPQBinByte, Vex);
          if PtrToLong(Vex) = 0 then Abort('BPQ Code is not loaded');

          Rex.AH := 0;                     {Verify that the INT is BPQ}
          Rex.AL := 1;
          Intr(BPQBinByte, Rex);
          if (Chr(Rex.AL)<>'B') or (Chr(Rex.AH)<>'P')
             or (Chr(Rex.BL)<>'Q') or (Chr(Rex.BH)<>' ') then
                Abort('BPQ Code is not loaded');

      end;



begin
                                           {signon message}
   WriteLn(ProgName, ^M^J, Copyright, ^M^J);

                                          {interpret command line parameters}
   if ParamCount = 0 then Abort(' Must supply parameters. Use /? for help');
   if ParamCount > 1 then Abort(' Only one parameter needed on command line!');
   Opt := ParamStr(1);

   CheckBPQInterrupt;                      {Verify BPQ is up and running}
   Initialize(Opt);                        {Determine what is requested}


                                 {Grab interrupt vectors and set up hot keys}
   if not InitVector(BPQBinByte, BPQSloDXHandle, @BPQScreen) or
    not DefinePop(ToggleScrnKey, ToggleScreening, Ptr(SSeg, SPtr), True) then
          Abort('Unable to install BPQSloDX');

                                           {Activate popups and go resident}
   PopupsOn;
   StayRes(ParagraphsToKeep, 0);

                                           {if we get here we failed}
   Abort('Unable to go resident');
end.
