
{$R-,V-,S-,B-,I-}                 {Turn off all the checking}
{$M 1024,100,100}                 {Small stack, no heap}

{$I OPDEFINE.INC}

program BPQUpCnt;          {Utility for uping hop count for DX spots operations}

uses
   Dos, OpInline, OpInt, OpTsr;

type
   String80 = string[80];

const
   BPQUpCntHandle = 16;                     {ISR handle for int 7F}
   ToggleScrnKey = $042E;                  {Toggle screening ctrl C}

   {external functions}
   UnloadProgram = Ord('U');

   ModuleName : string[8] = 'BPQUpCnt';     {module name}
   Version    = '1.10';

   ProgName : String[35] = 'BPQUpCnt '+Version+ ' BPQ Hop Count Routine';
   Copyright : String[37] = 'Written by Martin Schick, KA4IWG 1996';

   ScreeningIsActive : Boolean = True;    {Screening is off at startup}

   Pattern : Array[1..4] of byte = (94, 72, 49, 94);  {Hop count pattern}
   PatternCount = 4;

var
   NewCnt : byte;
   BPQBinByte : byte;
   Opt : String80;

procedure ToggleFlag(var Flag : Boolean);  {Toggle any flag}
   begin
      PopupsOff;                              {Shut off popups}
      Flag := not Flag;
      PopupsOn;                             {Turn popups back on}
   end;


{$F+}
procedure ToggleScreening(var Regs :Registers);
                               {Toggle screening - activated by <Ctrl><Tab>}
   begin
      ToggleFlag(ScreeningIsActive);
      if ScreeningIsActive then
          Writeln(' Hop screening is active')
      else
          Writeln(' Hop screening is inactive');
   end;


procedure BPQScreen(BP : Word); interrupt; {ISR called on every INT $7F}
   type
      str255 = string[255];
   var
      Regs : IntRegisters absolute BP;
      SaveRegs : IntRegisters;
      Vector : Pointer;
      I, J : Integer;
      Match : Boolean;
      Offset : Word;

   begin
      if ScreeningIsActive and (Regs.AH = $03) then
          begin
              EmulateInt(Regs, IsrArray[BPQUpCntHandle].OrigAddr);

              if Regs.CX > 0 then
              begin
                  with Regs do
                  begin
                      SaveRegs := Regs;
                  end;

                  for I := 0 to Regs.CX do
                  begin
                      if Pattern[1] = Mem[Regs.ES : Regs.DI] then
                      begin
                      {Check rest of Pattern}
                          Match := True;
                          Offset := Regs.DI;
                          for J := 1 to PatternCount do
                          begin
                              if Pattern[J] <> Mem[Regs.ES : Offset] then Match := False;
                              Inc(Offset);
                          end;

                          if Match then
                          begin
                              Offset := Regs.DI;      {Point to caret}
                              Inc(Offset);            {Point to H}
                              Inc(Offset);            {Point to count}
                              Mem[Regs.ES : Offset] := NewCnt;  {Put in new hop}
                          end;
                  end;
                  Inc(Regs.DI);
              end;

              with Regs do
              begin
                  Regs := SaveRegs;
              end;
          end;
          IntReturn(Regs);
      end
      else
          ChainInt(Regs, IsrArray[BPQUpCntHandle].OrigAddr)
   end;


procedure Unload(var Regs : Registers);
                                {Unload from memory - activated by <Ctrl><U>}
   var
   Cur7F, SaveRetVec, P : Pointer;

   begin
      WriteLn('Unloading BPQUpCnt from memory');
                                         {save copy of current return vector}
      SaveRetVec := GetReturnVec(BPQBinByte);

                                           {set new return vector}
      SetVecOnReturn(BPQBinByte, IsrArray[BPQUpCntHandle].OrigAddr);

                                    {is the current INT $7F vector in OPTSR?}
      GetIntVec(BPQBinByte, Cur7F);
      P := @DisableTSR;
      if OS(Cur7F).S = OS(P).S then
                                      {make program's ISR the current vector}
          SetIntVec(BPQBinByte, IsrArray[BPQUpCntHandle].NewAddr);

                                           {try to disable the TSR}
          if DisableTSR then begin

              ScreeningIsActive := False;

          end
      else begin
                                         {restore the previous return vector}
          SetVecOnReturn(BPQBinByte, SaveRetVec);

                                           {restore the previous ISR}
          SetIntVec(BPQBinByte, Cur7F);
      end;
   end;
{$F-}


procedure ExternalIFC(BP : Word); interrupt;
                                 {This ISR interfaces with the outside world}
   var
      Regs : IntRegisters absolute BP;
      SavePSP : Word;

   begin
                                           {interrupts on}
      InterruptsOn;

                                          {turn popups off}
      PopupsOff;

      with Regs do
      case AH of
          UnloadProgram :                  {unload BPQUpCnt}
          begin
                                           {save current PSP}
              SavePSP := GetPSP;

                                           {switch back to our PSP}
              SetPSP(PrefixSeg);

                                           {try to unload BPQUpCnt}
              AL := Ord(DisableTSR);
              if AL = 1 then begin

                  ScreeningIsActive := False;
              end;

                                           {switch back to previous PSP}
              SetPSP(SavePSP);
          end;
      end;

      PopupsOn;
   end;


procedure DisableResidentCopy(IP : IfcPtr);
                                      {-Disable resident copy of the program}
   var
      Regs : IntRegisters;
   begin
                                           {restore interrupt vectors}
      RestoreAllVectors;

                                           {make the function call}
      Regs.AH := UnloadProgram;
      EmulateInt(Regs, IP^.CmdEntryPtr);
      if Boolean(Regs.AL) then begin
          WriteLn(ModuleName, ' unloaded');
      end
      else
          WriteLn('Unable to unload ', ModuleName);

          Halt;
   end;


procedure Abort(Msg : String80);
                                           {-Write a message and halt}
   begin
      WriteLn(Msg);
      Halt(1);
   end;


procedure ShowHelp;                        {Help screen}

   begin
        WriteLn('Usage: ',ModuleName,' [Options]'^M^J);
        WriteLn('Options:');
        WriteLn('/$digit or -$digit            up hop count to digit counts');
        WriteLn('/? or -?                      this help screen');
        WriteLn('/U or -U                      unload ',ModuleName);
        WriteLn;
        WriteLn('<ctrl><C>       used to toggle screening off/on');
        Halt(0)
   end;


procedure Initialize(var Opt : String80);
                          {-Initialize and check for command line parameters}
   var
      ModPtr : IfcPtr;


   procedure InvalidOption;
                                         {-displays error message and aborts}
      begin
          WriteLn(Opt, ' is an invalid option'^M^J);
          WriteLn('Enter BPQUPCNT /? for help');
          Halt(0);
      end;


   begin
                                           {see if we're already resident}
      ModPtr := ModulePtrByName(ModuleName);

          if ((Opt[1] = '/') or (Opt[1] = '-')) and (Length(Opt) >= 2) then
          case UpCase(Opt[2]) of

              '$' : NewCnt := Ord(Opt[3]);

              '?' : ShowHelp;

              'U' : if ModPtr = nil then
                        Abort('Program is not resident. Cannot unload.')
                    else
                        DisableResidentCopy(ModPtr);
          else InvalidOption;
          end

      else
          InvalidOption;


                                           {abort if we're already resident}
      if ModPtr <> nil then
          Abort('Program is already installed');

                                           {install our module}
          InstallModule(ModuleName, @ExternalIFC);
          WriteLn('     Program installed');
   end;

   procedure CheckBPQInterrupt;
      var
          BPQConfig : File of byte;
          NN : integer;
          Vex : Pointer;
          Rex : Registers;

      begin                                {Try to open BPQCFG.BIN}
          Assign(BPQConfig, 'BPQCFG.BIN');
          {$I-}reset(BPQConfig);{$I+}
          NN := IOResult;
          if NN<>0 then Abort('BPQCFG.BIN not found');

          Seek(BPQConfig, 70);             {Find which INT BPQ is using}
          Read(BPQConfig, BPQBinByte);
          WriteLn('BPQ utilizes interrupt number ',BPQBinByte);
          WriteLn;
                                           {Check that vector for address}
          GetIntVec(BPQBinByte, Vex);
          if PtrToLong(Vex) = 0 then Abort('BPQ Code is not loaded');

          Rex.AH := 0;                     {Verify that the INT is BPQ}
          Rex.AL := 1;
          Intr(BPQBinByte, Rex);
          if (Chr(Rex.AL)<>'B') or (Chr(Rex.AH)<>'P')
             or (Chr(Rex.BL)<>'Q') or (Chr(Rex.BH)<>' ') then
                Abort('BPQ Code is not loaded');

      end;



begin
                                           {signon message}
   WriteLn(ProgName, ^M^J, Copyright, ^M^J);

                                          {interpret command line parameters}
   if ParamCount = 0 then Abort(' Must supply parameters. Use /? for help');
   if ParamCount > 1 then Abort(' Only one parameter needed on command line!');
   Opt := ParamStr(1);

   CheckBPQInterrupt;                      {Verify BPQ is up and running}
   Initialize(Opt);                        {Determine what is requested}


                                 {Grab interrupt vectors and set up hot keys}
   if not InitVector(BPQBinByte, BPQUpCntHandle, @BPQScreen) or
      not DefinePop(ToggleScrnKey, ToggleScreening, Ptr(SSeg, SPtr), True) then
          Abort('Unable to install BPQUpCnt');

                                           {Activate popups and go resident}
   PopupsOn;
   StayRes(ParagraphsToKeep, 0);

                                           {if we get here we failed}
   Abort('Unable to go resident');
end.
